#include <qcheckbox.h>
#include <qlineedit.h>

#include "../UI/HGLSplatRendererConfigurationWidget.h"
#include "HGLSplatRendererConfiguration.h"


HGLSplatRendererConfiguration::HGLSplatRendererConfiguration()
 : cutOffRadius(1.0f), conditionThreshold(7000.f), depthOffset(0.2f), programDirectory("Renderers/preview/cgShaders/perspCorrectShaders")
{
	configurationWidget = 0;
}


HGLSplatRendererConfiguration::~HGLSplatRendererConfiguration() 
{
}


void HGLSplatRendererConfiguration::setCutOffRadius (const float newCutOffRadius) 
{
  cutOffRadius = newCutOffRadius;
  // update the configuration widget, if available yet
  if (configurationWidget != 0) {
      configurationWidget->cutOffRadiusLineEdit->setText (QString::number (newCutOffRadius));
  }
  emit cutOffRadiusChanged (newCutOffRadius);
}


float HGLSplatRendererConfiguration::getCutOffRadius() const 
{
  return cutOffRadius;
}


void HGLSplatRendererConfiguration::setDepthOffset(const float newDepthOffset) 
{
	depthOffset = newDepthOffset;
	 if (configurationWidget != 0) {
      configurationWidget->depthOffsetLineEdit->setText (QString::number (newDepthOffset));
  }
  emit depthOffsetChanged (newDepthOffset);
}

float HGLSplatRendererConfiguration::getDepthOffset() const
{
	return depthOffset;
}


void HGLSplatRendererConfiguration::setConditionThreshold(const float newConditionThreshold) {
	conditionThreshold = newConditionThreshold;
	if (configurationWidget != 0) {
		configurationWidget->conditionThresholdLineEdit->setText (QString::number (newConditionThreshold));
	}
	emit conditionThresholdChanged(newConditionThreshold);
}


float HGLSplatRendererConfiguration::getConditionThreshold() const {
	return conditionThreshold;
}

QString HGLSplatRendererConfiguration::getCgProgramDirectory() const {
	return programDirectory;
}


QWidget *HGLSplatRendererConfiguration::createConfigurationWidget (QWidget *parent) {	
	configurationWidget = new HGLSplatRendererConfigurationWidget_Impl (parent);

	this->connect (configurationWidget, SIGNAL (reloadCGPrograms()),
				 this, SLOT (handleReloadCGPrograms()));

	// initialize the configuration widget
	configurationWidget->cutOffRadiusLineEdit->setText (QString::number (cutOffRadius));
	configurationWidget->conditionThresholdLineEdit->setText (QString::number (conditionThreshold));
	configurationWidget->depthOffsetLineEdit->setText (QString::number (depthOffset));
	configurationWidget->blendingCheckBox->setChecked(true);
	configurationWidget->cgProgramDirectoryLineEdit->setText(programDirectory);

	return configurationWidget;
}

void HGLSplatRendererConfiguration::apply() {

	float newCutOffRadius;
	float newConditionThreshold;
	float newDepthOffset;
	bool  isNumber;

	newCutOffRadius = configurationWidget->cutOffRadiusLineEdit->text().toFloat (&isNumber);
	if (isNumber == false) {
		// reset value
		configurationWidget->cutOffRadiusLineEdit->setText (QString::number (cutOffRadius));
	}
	else if (newCutOffRadius != cutOffRadius) {
		cutOffRadius = newCutOffRadius;
		emit cutOffRadiusChanged (newCutOffRadius);
	}

	newConditionThreshold = configurationWidget->conditionThresholdLineEdit->text().toFloat (&isNumber);
	if (isNumber == false) {
		// reset value
		configurationWidget->conditionThresholdLineEdit->setText (QString::number (conditionThreshold));
	}
	else if (newConditionThreshold != conditionThreshold) {
		conditionThreshold = newConditionThreshold;
		emit conditionThresholdChanged (newConditionThreshold);
	}

	newDepthOffset = configurationWidget->depthOffsetLineEdit->text().toFloat (&isNumber);
	if (isNumber == false) {
		// reset value
		configurationWidget->depthOffsetLineEdit->setText (QString::number (depthOffset));
	}
	else if (newDepthOffset != depthOffset) {
		depthOffset = newDepthOffset;
		emit depthOffsetChanged (newDepthOffset);
	}

	programDirectory = configurationWidget->cgProgramDirectoryLineEdit->text();

	emit setBlendingEnabled(configurationWidget->blendingCheckBox->isChecked());
	emit configurationApplied();
}

void HGLSplatRendererConfiguration::handleReloadCGPrograms() {
	emit reloadCGPrograms();
}
